// 背景執行		forever start -c ts-node -a -l canvas.log src/app.ts
// 重新背景執行	forever restart -a -l canvas.log src/app.ts
// 監聽檔案變化	"npm start"
// 連線Debug	"npm run dev"

import dayjs from "dayjs";
import "dayjs/locale/zh-tw";
import dotenv from "dotenv";

import dateFormat from "dateformat";
import express from 'express';
import fs from "fs";
import https from 'https';
import multer from 'multer';
import { generateImage } from "./canvas";

dayjs.locale("zh-tw");
if (process.env.NODE_ENV) {
	dotenv.config({ path: `./.env.${process.env.NODE_ENV}` });
} else {
	dotenv.config();
}

const path: string = process.env.URLPATH || "/";
const port: number = +process.env.PORT || 3000;
const app = express();
const upload = multer({ dest: `${path}/` }); // 用于处理文件上传

//讀取憑證及金鑰
const prikey: string = fs.readFileSync(process.env.prikeyPath, "utf8");
const cert: string = fs.readFileSync(process.env.certPath, "utf8");
const cafile: string = fs.readFileSync(process.env.cafilePath, "utf-8");

//建立憑證及金鑰
const credentials: Object = {
	key: prikey,
	cert: cert,
	ca: cafile
};

// 处理 POST 请求并返回生成的图片
app.post(`/${path}`, upload.single('file'), (req, res) => {
	const imageBuffer = generateImage();
	res.set('Content-Type', 'image/png');
	res.send(imageBuffer);
});

// 创建 HTTPS 服务器
https.createServer(credentials, app).listen(port, () => {
	let dateTime: string = dateFormat(new Date(), "yyyy-mm-dd HH:MM:ss");
	console.log(`${dateTime} listening on ${port} path: ${path}`);
	console.log(`${dateTime} [api已準備就緒]`);
});