import { createCanvas } from 'canvas';

// 生成图片的函数
export function generateImage() {
	const boys = ['男生A', '男生B', '男生C'];
	const girls = ['女生A', '女生B', '女生C'];
	const rounds = 3; // 可以根据需要设置轮数
	const person = boys.length > girls.length ? boys.length : girls.length;
	if (boys.length > girls.length) {
		girls.push("那個")
	} else if (boys.length < girls.length) {
		boys.push("那個")
	}
	const canvasWidth = 800; // 根据需要调整宽度
	const canvasHeight = (rounds * 100) + (person * 10); // 根据需要调整高度
	const canvas = createCanvas(canvasWidth, canvasHeight);
	const ctx = canvas.getContext('2d');
	const font = "PMingLiU";

	// 设定背景颜色
	ctx.fillStyle = 'white';
	ctx.fillRect(0, 0, canvas.width, canvas.height);

	// 追踪已使用的组合
	let usedCombinations = new Set<string>();

	// 生成不重复的队伍组合并确保每人上场
	function generateTeams(): { teamA: string, teamB: string }[] {
		const roundTeams: { teamA: string, teamB: string }[] = [];
		const availableBoys = [...boys];
		const availableGirls = [...girls];
		let attempts = 0;

		while (roundTeams.length < boys.length) {
			const boyIndex = Math.floor(Math.random() * availableBoys.length);
			const girlIndex = Math.floor(Math.random() * availableGirls.length);
			const team = `${availableBoys[boyIndex]}-${availableGirls[girlIndex]}`;

			// 如果组合未使用过，添加到队伍列表
			if (!usedCombinations.has(team)) {
				roundTeams.push({
					teamA: availableBoys[boyIndex],
					teamB: availableGirls[girlIndex]
				});
				usedCombinations.add(team);
				availableBoys.splice(boyIndex, 1);
				availableGirls.splice(girlIndex, 1);
			}

			// 如果尝试过多次未找到合适组合，强制生成剩余队伍
			attempts++;
			if (attempts > 10 && roundTeams.length < boys.length) {
				for (let i = 0; i < availableBoys.length; i++) {
					roundTeams.push({
						teamA: availableBoys[i],
						teamB: availableGirls[i % availableGirls.length]
					});
				}
				break;
			}
		}

		return roundTeams;
	}

	// 生成所有轮次的队伍组合
	let teamCombinations: { teamA: string, teamB: string }[][] = [];
	for (let i = 0; i < rounds; i++) {
		const newRound = generateTeams();
		teamCombinations.push(newRound);

		// 如果所有组合都用过了，重置组合记录
		if (usedCombinations.size >= boys.length * girls.length) {
			usedCombinations.clear();
		}
	}

	// 绘制标题和日期
	ctx.fillStyle = 'black';
	ctx.font = `bold 24px "${font}"`;
	ctx.textAlign = 'center';
	ctx.fillText('比賽隊伍', canvasWidth / 2, 40);

	ctx.font = `italic 16px "${font}"`;
	ctx.fillText('2024-09-09', canvasWidth / 2, 70);

	const roundTitleYStart = 120;
	const teamYStart = 60;
	const lineHeight = 30;
	const roundsPerRow = 3;
	const rowHeight = 300;

	teamCombinations.forEach((roundTeams, roundIndex) => {
		const rowIndex = Math.floor(roundIndex / roundsPerRow);
		const colIndex = roundIndex % roundsPerRow;
		const roundXPosition = (canvasWidth / roundsPerRow) * colIndex + (canvasWidth / roundsPerRow) / 2;
		const roundYPosition = roundTitleYStart + rowIndex * rowHeight;

		ctx.font = `bold 18px "${font}"`;
		ctx.textAlign = 'center';
		ctx.fillText(`第 ${roundIndex + 1} 輪`, roundXPosition, roundYPosition);

		ctx.font = `16px "${font}"`;

		roundTeams.forEach((team, teamIndex) => {
			const teamYPosition = roundYPosition + teamYStart + teamIndex * lineHeight;
			if (teamIndex === 0) {
				ctx.fillText('1號隊友', roundXPosition - 50, teamYPosition);
				ctx.fillText('2號隊友', roundXPosition + 50, teamYPosition);
			}
			ctx.fillText(team.teamA, roundXPosition - 50, teamYPosition + lineHeight);
			ctx.fillText(team.teamB, roundXPosition + 50, teamYPosition + lineHeight);
		});
	});

	// 导出图片
	const buffer = canvas.toBuffer('image/png');
	// fs.writeFileSync('./team_combination.png', buffer);
	console.log('圖片已生成！');
	return buffer;
}